/*
 * MegaLoad low level functions.
 */

#include <avr/io.h>

//*********************************************************

workreg  = 18
wr0      = 0
wr1      = 1
wr30     = 30
par1     = 24
par2     = 22
par1h    = 25

#if __AVR_MEGA__
  #define XJMP jmp
  #define XCALL call
#else
  #define XJMP rjmp
  #define XCALL rcall
#endif

#ifndef SPMCR
 #define SPMCR SPMCSR
#endif
//-----------------------------------------

// void write_page (unsigned int adr, unsigned char function);
// bits 8:15 adr addresses the page...(must setup RAMPZ beforehand!!!)
.global write_page
.func write_page
write_page:
    XCALL __WAIT_SPMEN__
    movw    wr30, par1        // move address to z pointer (R31 = ZH, R30 = ZL)
#if _SFR_IO_REG_P(SPMCR)
    out     _SFR_IO_ADDR(SPMCR), par2
#else
    sts     _SFR_MEM_ADDR(SPMCR), par2
#endif
    SPM                     // perform pagewrite
    RET
.endfunc

//-----------------------------------------

// void fill_temp_buffer (unsigned int data, unsigned int adr);
// bits 7:1 in adr addresses the word in the page... (2=first word, 4=second word etc..)
.global fill_temp_buffer
.func fill_temp_buffer
fill_temp_buffer:
    XCALL __WAIT_SPMEN__
    movw    wr30, par2      // move adress to z pointer (R31=ZH R30=ZL)
    movw    wr0, par1       // move data to reg 0 and 1
    LDI     workreg, 0x01
#if _SFR_IO_REG_P(SPMCR)
    out     _SFR_IO_ADDR(SPMCR), workreg
#else
    sts     _SFR_MEM_ADDR(SPMCR), workreg
#endif
    SPM                   // Store program memory
    clr       wr1
    RET
.endfunc

//-----------------------------------------

// unsigned int read_program_memory (unsigned int adr ,unsigned char cmd);
.global read_program_memory
.func read_program_memory
read_program_memory:
    movw    wr30, par1        // move adress to z pointer
    SBRC    par2, 0           // read lockbits? (second argument = 0x09)
#if _SFR_IO_REG_P(SPMCR)
    out     _SFR_IO_ADDR(SPMCR), par2
#else
    sts     _SFR_MEM_ADDR(SPMCR), par2
#endif
#if defined (__AVR_ATmega128__)|| defined (__AVR_ATmega1280__)
    ELPM    par1, Z+          // read LSB
    ELPM    par1h, Z           // read MSB
#else
    LPM     par1, Z+
    LPM     par1h, Z
#endif
    RET
.endfunc

//-----------------------------------------

#if defined (LOCKBIT)
// void write_lock_bits (unsigned char val);
.global write_lock_bits
.func write_lock_bits
write_lock_bits:
     MOV     wr0, par1
     LDI     workreg, 0x09
#if _SFR_IO_REG_P(SPMCR)
    out     _SFR_IO_ADDR(SPMCR), workreg
#else
    sts     _SFR_MEM_ADDR(SPMCR), workreg
#endif
     SPM                    // write lockbits
     RET
.endfunc
#endif
//-----------------------------------------

.global enableRWW
.func enableRWW
enableRWW:
    XCALL __WAIT_SPMEN__
    LDI workreg, 0x11
#if _SFR_IO_REG_P(SPMCR)
    out     _SFR_IO_ADDR(SPMCR), workreg
#else
    sts     _SFR_MEM_ADDR(SPMCR), workreg
#endif
    SPM
    RET
.endfunc

//-----------------------------------------
.func __WAIT_SPMEN__
__WAIT_SPMEN__:
    lds     workreg, _SFR_MEM_ADDR(SPMCR)
    SBRC    workreg, 0           // check SPMEN flag
    RJMP    __WAIT_SPMEN__       // wait for SPMEN flag cleared
    RET
.endfunc
//-----------------------------------------
